// InfoVis - Baumvisualisierung in OpenGL 2D + 3D
//
#include "support.h"
#include <dirent.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <errno.h>
#include <vector>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>

using namespace std;

class File {
public:
	const char* path;
	const char* name;
	bool isDir;
	int randSize;
	vector<File> children;
	void addChild(File file);
	File();
	File(char* path2file);
	File(char* path2file, char* theName);
};
File::File() {
	path = "swap";
	name = "noname";

	support rand = support();
	randSize = rand.RandInt(90);
}
File::File(char* path2file) {
	path = path2file;
	name = path;

	support rand = support();
	randSize = rand.RandInt(90);
}
File::File(char* path2file, char* theName) {
	path = path2file;
	name = theName;
	
	support rand = support();
	randSize = rand.RandInt(90);
}

void File::addChild(File file) {
	children.push_back(file);
}

// Start time, current time.
double startT = support::getTime();

// Screen widths and heights (set by initialization code, 
// since reshape is called first).
int screenWidth2D;
int screenHeight2D;
int screenWidth3D;
int screenHeight3D;

// Mouse viewing rotation points
const double PI = 3.1415926;
const double DEG_TO_RAD = PI / 180.;
const double MIN_PHI = 1., MAX_PHI = 89.;
const double MIN_RADIUS = 3., MAX_RADIUS = 20.;
double mouse_theta = 0., mouse_phi = 45., mouse_radius = 5.;

// Mouse rotation modes
enum { MOUSE_NONE, MOUSE_ROTATE, MOUSE_ZOOM };
const double ROTATE_RATE = 0.75;
const double ZOOM_RATE = 0.025;
int mouse_mode = MOUSE_NONE;
int mouse_lastx, mouse_lasty;

// World ranges (in positive _and_ negative directions)
const double GROUND_RANGE = 3.;  // Ground size in 3-D
const double PLANT_RANGE = 2.0;  // Range of areas to plant trees over
const double APP_START_TIME = support().getTime();

// GLUT windows
int window2D, window3D;

// Forward Declarations
void myMouse(int button, int state, int mousex, int mousey);
void myMovedMouse(int mousex, int mousey);
void myKey(unsigned char key, int x, int y);
void myIdle(void);
void my2DDisplay(void);
void my2DReshape(int w, int h);
void my3DDisplay(void);
void my3DReshape(int w, int h);
void myInit(void);
void Draw(void);
void Draw2D(void);
void Draw3D(void);
void drawBranch(bool threeD, File dir, double size);
void mkDirTree(File dir);
File traverse(File dir2search, int indent);

File root;

void mkDirTree(File dir, int indent) {
	cout << dir.name << "\n";

	vector<File>::iterator e = dir.children.begin();
	while(e != dir.children.end() ) {
		for(int i=0;i<indent;i++)
			cout << "\t";
		mkDirTree(*e, indent+1);
		e++;
	}
}

// Main function -- starts main loop and executes initialization code
int main(int argc, char* argv[]) {
	if (argc != 2) {
		cerr << "Wrong number of arguments\n" << "Usage: fio ";
		return 1;
	}
	
	// get started;
	File start = File(argv[1]);
	start.isDir = true;
	root = traverse(start, 0);
/*
	root.addChild(File("a"));
	root.addChild(File("b"));
	root.addChild(File("c"));
	(root.children.at(0)).addChild(File("ChildOfA"));
*/

	cout << "All Files\n";
	mkDirTree(root,0);
	
	
	
	
  glutInit(&argc, argv);
  
  // 2D Setup
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
  glutInitWindowSize(400, 300);
  glutInitWindowPosition(50, 100);
  window2D=glutCreateWindow("2D Tree");
  glutDisplayFunc(my2DDisplay);
  glutReshapeFunc(my2DReshape);
  glutIdleFunc(myIdle);
  glutKeyboardFunc(myKey);

  // 3D Setup
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_DEPTH | GLUT_RGB);
  glutInitWindowSize(300, 300);
  glutInitWindowPosition(500, 100);
  window3D=glutCreateWindow("3D Tree");
  glutDisplayFunc(my3DDisplay);
  glutReshapeFunc(my3DReshape);
  glutIdleFunc(myIdle);
  glutMouseFunc(myMouse);
  glutMotionFunc(myMovedMouse);
  glutKeyboardFunc(myKey);

  // Initialize
  myInit();

  // Start main loop
  glutMainLoop();	
	
	return 0;
}


// mostly from C/C++ Language Support User's Guide
// http://publib.boulder.ibm.com/infocenter
File traverse(File startDir, int indent) {
	DIR *dir;
	struct dirent *entry;
	char address[1025];
	struct stat info;

	if ((dir = opendir(startDir.path)) == NULL)
		perror("opendir() error");
	else {
		while ((entry = readdir(dir)) != NULL) {
			if(entry->d_name[0] != '.') {
				strcpy(address, startDir.path);
				strcat(address, "/");
				strcat(address, entry->d_name);
				// File found
				File found = File(address,entry->d_name);
				if (stat(address, &info) != 0)
					fprintf(stderr, "stat() error on %s: %s\n", address, strerror(errno));
				else if(S_ISDIR(info.st_mode)) {
					// found directory
					found.isDir = true;
					startDir.addChild(traverse(found, indent+1));
				}
				else {
					found.isDir = false;
					startDir.addChild(found);
				}
			}
		}
	closedir(dir);
	}
	return startDir;
}



/************************************************************************/
/*                Display Functions                                     */
/************************************************************************/

void drawLine(GLfloat x1, GLfloat y1, GLfloat x2, GLfloat y2) {
	glBegin(GL_LINES);
		glVertex2f(x1, y1);
		glVertex2f(x2, y2);
	glEnd();
}


// This function draws a tree which should be seen in the 2D window
void Draw2D(void) {
	support::RandSeed(root.randSize);
	glPushMatrix();
		drawBranch(false, root, 100);
	glPopMatrix();
}

// This function draws a tree which should be seen in the 3D window
void Draw3D(void) {
	glPushMatrix();
		drawBranch(true, root, 1);
	glPopMatrix();
}

void draw(File dir, double size) {

	GLUquadricObj *quad;
	quad = gluNewQuadric();
	gluQuadricDrawStyle(quad, GLU_FILL); // GLU_FILL, GLU_LINE, GLU_SILHOUETTE, and GLU_POINT
	gluQuadricNormals(quad, GLU_SMOOTH);

	int details = 8;
	glColor3d(0.5+support::RandDouble()/10,
			0.45+support::RandDouble()/10,
			0.3+support::RandDouble()/10);
	//glColor3d(0.5,0.45,0.3);
	glPushMatrix();
		// for 2D view
		glRectf(-size/10,0,size/10,size);
		glRotated(-90,1,0,0);
		gluCylinder(quad, size/6, size/10, size, details, details);
	glPopMatrix();
	glTranslated(0, size, 0);
	
	if(!dir.isDir) {
		// draw fruit
		glColor3d(support::RandDouble(),
			support::RandDouble(),
			support::RandDouble());
		gluSphere(quad, size/4, details, details);
		// for 2D view
		gluDisk(quad, 0, size/4, details, details);		
	}
	/*
	glColor3d(1,0,0);
	glTranslated(size/10, -size/10, 0);
	support::drawString(dir.name, 10);
	glTranslated(-size/10, size/10, 0);
	*/
}

void drawBranch(bool threeD, File dir, double size) {
	draw(dir, size);
	int i=0;
	//glRotated(90,0,0,1);
	vector<File>::iterator e = dir.children.begin();
	while(e != dir.children.end() ) {
		//glTranslated(0, -size/2/dir.children.size(), 0);
		glPushMatrix();
			if(threeD) glRotated(i*360/dir.children.size()+10*support::RandDouble(),0,1,0);
			glRotated(i*160/dir.children.size()+280,0,0,1);
			drawBranch(threeD, *e, size/(1.3+support::RandDouble()));
		glPopMatrix();
		e++;
		i++;
	}
}


// Mouse function -- on entry:
//   button = GLUT_LEFT_BUTTON, GLUT_MIDDLE_BUTTON, or GLUT_RIGHT_BUTTON
//   state =  GLUT_UP or GLUT_DOWN
//   x, y = mouse location
void myMouse(int button, int state, int mousex, int mousey) {
  int x = mousex;
  int y = screenHeight3D - mousey;

  // Catch left mouse button
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) {
      mouse_mode = MOUSE_ROTATE;
      mouse_lastx = x;
      mouse_lasty = y;
    }
    else
      mouse_mode = MOUSE_NONE;
  }

  // Catch right mouse button
  if (button == GLUT_RIGHT_BUTTON) {
    if (state == GLUT_DOWN) {
      mouse_mode = MOUSE_ZOOM;
      mouse_lastx = x;
      mouse_lasty = y;
    }
    else
      mouse_mode = MOUSE_NONE;
  }
}


// Mouse function -- on movement:
//   x, y = mouse location
void myMovedMouse(int mousex, int mousey) {
  int x = mousex;
  int y = screenHeight3D - mousey;

  // Update rotation or zoom (depending on mode)
  switch (mouse_mode) {
  case MOUSE_ROTATE:
    mouse_theta -= (double)(x - mouse_lastx) * ROTATE_RATE;
    if (mouse_theta < 0.0)
	    mouse_theta += 360.;
    if (mouse_theta >= 360.)
	    mouse_theta -= 360.;
    mouse_phi += (double)(y - mouse_lasty) * ROTATE_RATE;
    if (mouse_phi >= MAX_PHI)
	    mouse_phi = MAX_PHI;
    if (mouse_phi < MIN_PHI)
	    mouse_phi = MIN_PHI;
    break;
  case MOUSE_ZOOM:
    mouse_radius += (double)(y - mouse_lasty) * ZOOM_RATE;
    if (mouse_radius >= MAX_RADIUS)
	    mouse_radius = MAX_RADIUS;
    if (mouse_radius < MIN_RADIUS)
	    mouse_radius = MIN_RADIUS;
    break;
  };
  mouse_lastx = x;
  mouse_lasty = y;

  // Recompute lookat (spherical coords)
  double ex, ey, ez;
  ex = mouse_radius * cos(DEG_TO_RAD * mouse_theta) * sin(DEG_TO_RAD * mouse_phi); 
  ey = mouse_radius * sin(DEG_TO_RAD * mouse_theta) * sin(DEG_TO_RAD * mouse_phi); 
  ez = mouse_radius * cos(DEG_TO_RAD * mouse_phi); 
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(ey, ez, ex, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

  // Redraw
  glutPostRedisplay();
}


// Keyboard function -- edit to add extra modes
//   key = ASCII key value
//   x, y = mouse location where keypress occurred
void 
myKey(unsigned char key, int x, int y) {
  switch (key) {
  case 'q':
  case 'Q': exit(1); break;
  }
}


// Idle function 
void 
myIdle(void) {
  // Tell both windows to redisplay (this gives good animation)
  glutSetWindow(window2D);
  glutPostRedisplay();
  glutSetWindow(window3D);
  glutPostRedisplay();
}


// 2D Display function -- called to redraw window
// This function is specifically for the 2D window
void my2DDisplay(void) {
  // Clear screen
  glClear(GL_COLOR_BUFFER_BIT);
  
  // Draw tree and such
  Draw2D();

  // Show what we've drawn
  glutSwapBuffers();
}


// 3D Display function -- called to redraw window
// This function is specifically for the 3D window
void 
my3DDisplay(void) {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // Ground plane
  glColor3f(0.1, 0.2, 0.4);
  glBegin(GL_QUADS);
  glVertex3d(GROUND_RANGE, 0., GROUND_RANGE);
  glVertex3d(GROUND_RANGE, 0., -GROUND_RANGE);
  glVertex3d(-GROUND_RANGE, 0., -GROUND_RANGE);
  glVertex3d(-GROUND_RANGE, 0., GROUND_RANGE);
  glEnd();

  // Draw tree and such
  Draw3D();
  
  // Show what we've drawn
  glutSwapBuffers();
}


// 2D Reshape function -- called whenever window size changes
//   w, h = new window size
// This function is specifically for the 2D window
void my2DReshape(int width, int height)
{
  //printf("2D reshaped to %d %d\n",width,height);
  screenWidth2D=width;
  screenHeight2D=height;
  glViewport( 0, 0, screenWidth2D, screenHeight2D );
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(-(GLdouble)screenWidth2D * 0.5, (GLdouble)screenWidth2D * 0.5, 
             0.0, (GLdouble)screenHeight2D);
  glClear(GL_COLOR_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
}


// 3D Reshape function -- called whenever window size changes
//   w, h = new window size
// This function is specifically for the 3D window
void
my3DReshape(int w, int h) {
 // printf("3D reshaped to %d %d\n",w,h);
  screenWidth3D = w;
  screenHeight3D = h;
  glViewport(0, 0, screenWidth3D, screenHeight3D);	
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(50.0, (GLdouble)screenWidth3D / (GLdouble)screenHeight3D, 1.0, MAX_RADIUS + 5.);
  glMatrixMode(GL_MODELVIEW);
} 


// Initialization code
// This function initializes both windows
void 
myInit(void) {
  glutSetWindow(window2D);
  glClearColor(0.0, 0.0, 0.0, 0.0);

  glutSetWindow(window3D);
  glEnable(GL_DEPTH_TEST);
  glClearColor(0.0, 0.0, 0.0, 0.0);
  mouse_lastx = 0;
  mouse_lasty = screenHeight3D;
  mouse_mode = MOUSE_ROTATE;
  myMovedMouse(0, 0);
  mouse_mode = MOUSE_NONE;
}
