// Support Code
// This file implements utility code like the getTime() function.
//

#include "support.h"
#include <time.h>


// Uninitialized random numbers
int support::bRandInit = 0;


// Get the global time with at least millisecond resolution.
double 
support::getTime(void) {
#ifdef WIN32
  struct _timeb tbuf;
  _ftime(&tbuf);
  return ((double)tbuf.time + 0.001 * (double)(tbuf.millitm)); 
#else
  struct timeval time;
  gettimeofday(&time, NULL);
  return ((double)time.tv_sec + 0.000001 * (double)time.tv_usec); 
#endif
}


void support::drawString(const char* string, int height) {
  // The Mono Roman font supported by GLUT has characters that are
   // 104.76 units wide, up to 119.05 units high, and descenders can
   // go as low as 33.33 units.
   // We scale the text to make its height that desired by the caller.
   const static float FONT_HEIGHT = 119.05f;
   glPushMatrix();
      float s = height / FONT_HEIGHT; // scale factor
      glScalef( s, s, 1 );
      for ( int j = 0; string[j] != '\0'; ++j )
         glutStrokeCharacter( GLUT_STROKE_MONO_ROMAN, string[j] );
   glPopMatrix();
}

// Specify seed for random number generator
void 
support::RandSeed(int s) {
  srand((unsigned)s);
}


// Get random number between 0 and n - 1 (inclusive)
int 
support::RandInt(int n) {
  if (!bRandInit) {
    srand((unsigned)time(NULL));
    bRandInit = 1;
  }

  // For large numbers (> 10K) this will get less random
  return rand() % n;
}


// Get floating point random numbers between 0.0 and 1.0
double 
support::RandDouble(void) {
  if (!bRandInit) {
    srand((unsigned)time(NULL));
    bRandInit = 1;
  }

  return (double)rand() / (double)RAND_MAX; 
}